@@ -1,14 +1,15 @@
-# $Id: ParserFactory.pm,v 1.9 2002/01/28 19:35:27 matt Exp $
+# $Id: ParserFactory.pm,v 1.14 2008-08-04 04:51:29 grant Exp $
 
 package XML::SAX::ParserFactory;
 
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.00';
+$VERSION = '1.01';
 
 use Symbol qw(gensym);
 use XML::SAX;
+use XML::SAX::Exception;
 
 sub new {
     my $class = shift;
@@ -27,14 +28,14 @@ sub parser {
     
     my $parser_class = $self->_parser_class();
 
-    {
-        my $parser_file = $parser_class;
-        $parser_file =~ s/::/\//g;
-        $parser_file .= ".pm";
-        no strict 'refs';
-        if (!keys %{"${parser_class}::"}) {
-            require $parser_file;
-        }
+    my $version = '';
+    if ($parser_class =~ s/\s*\(([\d\.]+)\)\s*$//) {
+        $version = " $1";
+    }
+
+    if (!$parser_class->can('new')) {
+        eval "require $parser_class $version;";
+        die $@ if $@;
     }
 
     return $parser_class->new(@parser_params);
@@ -77,12 +78,12 @@ sub _parser_class {
             );
     }
 
-    # Next try ParserDetails.ini
+    # Next try SAX.ini
     for my $dir (@INC) {
         my $fh = gensym();
         if (open($fh, "$dir/SAX.ini")) {
             my $param_list = XML::SAX->_parse_ini_file($fh);
-            my $params = $param_list->[0];
+            my $params = $param_list->[0]->{Features};
             if ($params->{ParserPackage}) {
                 return $params->{ParserPackage};
             }
@@ -97,7 +98,7 @@ sub _parser_class {
                     }
                     return $parser->{Name};
                 }
-                throw XML::SAX::Exception ("Unable to provide SAX.ini required features");
+                XML::SAX->do_warn("Unable to provide SAX.ini required features. Using fallback\n");
             } 
             last; # stop after first INI found
         }
